@@ -2,6 +2,7 @@ use strict;
 use warnings;
 package App::DubiousHTTP::Tests;
 use App::DubiousHTTP::Tests::Common;
+use App::DubiousHTTP;
 use MIME::Base64 'encode_base64';
 
 my @cat;
@@ -130,6 +131,7 @@ sub auto {
     $html .= "accept = '".quotemeta($accept)."';\n" if $accept;
 
     $html .= "expect64 = '".encode_base64($body,'')."';\n";
+    $html .= 'results = "V | '.App::DubiousHTTP->VERSION.'\n";' . "\n";
     $isbad ||= '';
     $html .= "isbad ='$isbad';\n";
     $html .= "var checks = [];\n";
@@ -334,6 +336,11 @@ function check_page(req,test,status) {
     if (!status) {
 	status = req.status;
     }
+
+    var header;
+    try { header = req.getAllResponseHeaders(); } catch(e) {}
+    if (header == undefined || header == null) { header = ''; }
+
     if (!status) {
 	status = 'invalid';
     } else if (status >0) {
@@ -353,6 +360,7 @@ function check_page(req,test,status) {
 		status = 'match';
 	    } else {
 		status = 'change(' + status + ')';
+		results = results + "R | " + test['page'] + " | " + response.length + " | " + base64_encode((header + "--\n" + response).substr(0,1000)) + "\n";
 		_log( "len=" + response.length + "   " + test['page'] + ' - ' + test['desc'] );
 		_log( "response: " + result64 );
 		_log( "expect:   " + expect );
@@ -360,12 +368,9 @@ function check_page(req,test,status) {
 	}
     }
 
-    if (test['log_header']) {
+    if (test['log_header'] && header != '') {
 	// i.e. Via added or similar
-	try {
-	    var header = req.getAllResponseHeaders();
-	    results = results + "H | " + test['page'] + " | " + base64_encode(header) + "\n";
-	} catch(e) {}
+	results = results + "H | " + test['page'] + " | " + base64_encode(header) + "\n";
     }
 
     add_debug( test['desc'] + ' - ' + status + ( test['harmless_retry'] ? ' - retry with harmless content':''), test);
