@@ -28,7 +28,8 @@ DESC
 
     [ 'VALID: combined with content-length' ],
     # according to RFC2616 TE chunked has preference to clen
-    [ VALID, 'chunked,clen' => 'chunking and content-length, served chunked'],
+    [ VALID, 'chunked,clen' => 'chunked first then content-length, served chunked'],
+    [ VALID, 'clen,chunked' => 'content-length first then chunked, served chunked'],
     # but some still expect clen bytes
     [ VALID, 'chunked,clen200' => 'chunking and content-length header with double length, served chunked'],
     [ VALID, 'chunked,clen50'  => 'chunking and content-length header with half length, served chunked'],
@@ -36,7 +37,12 @@ DESC
     [ 'INVALID: chunking is only allowed with HTTP/1.1' ],
     [ INVALID, 'chunked,http10' => 'Chunked Header and HTTP/1.0. Served chunked.'],
     [ INVALID, 'chunked,clen,http10' => 'Chunked Header and Content-length and HTTP/1.0. Served chunked.'],
-    [ 'INVALID: chunking is only allowed with HTTP/1.1' ],
+    [ INVALID, 'clen,chunked,http10' => 'Content-length Header and Chunked and HTTP/1.0. Served chunked.'],
+
+    [ 'VALID: chunked header should be ignored with HTTP/1.0' ],
+    [ UNCOMMON_VALID, 'chunked,http10,do_clen' => 'Chunked Header and HTTP/1.0. Not served chunked.'],
+    [ UNCOMMON_VALID, 'chunked,clen,http10,do_clen' => 'Chunked Header and Content-length and HTTP/1.0. Not served chunked.'],
+    [ UNCOMMON_VALID, 'clen,chunked,http10,do_clen' => 'Content-length Header and Chunked and HTTP/1.0. Not served chunked.'],
 
     [ 'INVALID: chunking with invalid HTTP versions' ],
     [ INVALID, 'chunked,HTTP/1.2' => 'Chunked Header and HTTP/1.2. Served chunked.'],
@@ -58,8 +64,6 @@ DESC
 
     [ 'VALID: valid variations on "chunked" value' ],
     [ VALID, 'chUnked' => 'mixed case "chUnked", served chunked'],
-    [ VALID, 'rfc2047,do_clen' => 'rfc2047/base64 encoded "chunked", not served chunked' ],
-    [ VALID, 'rfc2047,clen,do_clen' => 'rfc2047/base64 encoded "chunked" and content-length, not served chunked' ],
     [ UNCOMMON_VALID,'nl-chunked' => "chunked header with continuation line, served chunked"],
     [ UNCOMMON_VALID,'nl-nl-chunked' => "chunked header with double continuation line, served chunked"],
 
@@ -70,15 +74,38 @@ DESC
     [ INVALID, 'chunkedx' => '"chunkedx" not "chunked"'],
     [ INVALID, 'chunked-x' => '"chunked x" not "chunked"'],
     [ INVALID, 'x-chunked' => '"x chunked" not "chunked"'],
+    [ UNCOMMON_INVALID, 'chunked-x,do_clen' => '"chunked x" not "chunked", not served chunked'],
+    [ UNCOMMON_INVALID, 'x-chunked,do_clen' => '"x chunked" not "chunked", not served chunked'],
     [ INVALID, 'x-nl-chunked' => '"x-folding-chunked" not "chunked"'],
     [ INVALID, 'rfc2047,do_chunked' => 'rfc2047/base64 encoded "chunked", served chunked' ],
+    [ UNCOMMON_VALID, 'rfc2047,do_clen' => 'rfc2047/base64 encoded "chunked", not served chunked' ],
+    [ UNCOMMON_VALID, 'rfc2047,clen,do_clen' => 'rfc2047/base64 encoded "chunked" and content-length, not served chunked' ],
+    [ INVALID,'nl-chunked,do_clen' => "chunked header with continuation line. Not served chunked."],
+    [ INVALID,'nl-nl-chunked,do_clen' => "chunked header with double continuation line, not served chunked"],
+    [ INVALID,'ce-chunked,do_chunked' => "Content-encoding chunked instead of Transfer-encoding. Served chunked."],
+
+    [ 'INVALID: hiding with another Transfer-Encoding' ],
     [ INVALID, 'xte,chunked,do_chunked' => "double Transfer-Encoding: first junk, last chunked. Served chunked." ],
     [ INVALID, 'chunked,xte,do_chunked' => "double Transfer-Encoding: first chunked, last junk. Served chunked." ],
+    [ INVALID, 'xte,chunked,xte,do_chunked' => "triple Transfer-Encoding: first junk, then chunked, then junk again. Served chunked." ],
+    [ INVALID, 'xte,chunked,do_clen' => "double Transfer-Encoding: first junk, last chunked. Not served chunked." ],
+    [ INVALID, 'chunked,xte,do_clen' => "double Transfer-Encoding: first chunked, last junk. Not served chunked." ],
     [ INVALID, 'chunked,xte,clen,do_chunked' => "double Transfer-Encoding: first chunked, last junk. Also Content-length header. Served chunked." ],
     [ INVALID, 'xte,chunked,clen,do_chunked' => "double Transfer-Encoding: first junk, last chunked. Also Content-length header. Served chunked." ],
+    [ INVALID, 'xte,chunked,xte,clen,do_chunked' => "triple Transfer-Encoding: first junk, then chunked, then junk again. Also Content-length header. Served chunked." ],
+    [ INVALID, 'chunked,xte,clen,do_clen' => "double Transfer-Encoding: first chunked, last junk. Also Content-length header. Not served chunked." ],
+    [ INVALID, 'xte,chunked,clen,do_clen' => "double Transfer-Encoding: first junk, last chunked. Also Content-length header. Not served chunked." ],
     [ INVALID, 'chunked,clen,do_clen' => 'chunking and content-length, not served chunked'],
-    [ INVALID,'nl-chunked,do_clen' => "chunked header with continuation line. Not served chunked."],
-    [ INVALID,'ce-chunked,do_chunked' => "Content-encoding chunked instead of Transfer-encoding. Served chunked."],
+
+    [ 'INVALID: hiding the Transfer-Encoding header' ],
+    [ INVALID, 'space-colon-chunked,do_chunked' => '"Transfer-Encoding<space>:", served chunked' ],
+    [ UNCOMMON_INVALID, 'space-colon-chunked,do_clen' => '"Transfer-Encoding<space>:", not served chunked' ],
+    [ INVALID, 'colon-colon-chunked,do_chunked' => '"Transfer-Encoding::", served chunked' ],
+    [ UNCOMMON_INVALID, 'colon-colon-chunked,do_clen' => '"Transfer-Encoding::", not served chunked' ],
+    [ INVALID, 'cronly-chunked,do_chunked' => 'Transfer-Encoding with only <CR> as line delimiter before, served chunked' ],
+    [ UNCOMMON_INVALID, 'cronly-chunked,do_clen' => 'Transfer-Encoding with only <CR> as line delimiter before, not served chunked' ],
+    [ UNCOMMON_INVALID, 'lfonly-chunked,do_chunked' => 'Transfer-Encoding with only <LF> as line delimiter before, served chunked' ],
+    [ INVALID, 'lfonly-chunked,do_clen' => 'Transfer-Encoding with only <LF> as line delimiter before, not served chunked' ],
 
     [ 'INVALID: invalid chunks' ],
     [ INVALID, 'chunked-lf' => "chunk with LF as delimiter instead of CRLF" ],
@@ -94,16 +121,22 @@ sub make_response {
     for (split(',',$spec)) {
 	if ( ! $_ || $_ eq 'chunked' ) {
 	    $hdr .= "Transfer-Encoding: chunked\r\n"
+	} elsif ( $_ eq 'space-colon-chunked' ) {
+	    $te = 'chunked';
+	    $hdr .= "Transfer-Encoding : chunked\r\n"
+	} elsif ( $_ eq 'colon-colon-chunked' ) {
+	    $te = 'chunked';
+	    $hdr .= "Transfer-Encoding:: chunked\r\n"
+	} elsif ( my ($crlf) = m {^(lf|cr)only-chunked$} ) {
+	    $te = 'chunked';
+	    $hdr = "X-Foo: bar" if $hdr !~s{\r\n\z}{};
+	    $hdr .= ($crlf eq 'lf') ? "\n":"\r";
+	    $hdr .= "Transfer-Encoding: chunked\r\n";
 	} elsif ( $_ eq '1chunk' ) {
 	    $hdr .= "Transfer-Encoding: chunked\r\n";
 	    $te = $_
 	} elsif ( $_ eq 'chUnked' ) {
 	    $hdr .= "Transfer-Encoding: chUnked\r\n"
-	} elsif ( m{^(.*-)?nl-(nl-)?chunked$} ) {
-	    my $prefix = $1 ||'';
-	    $hdr .= "Transfer-Encoding: $prefix\r\n ";
-	    $hdr .= "\r\n " if $2;
-	    $hdr .= "chunked\r\n"
 	} elsif ( $_ eq 'chu' ) {
 	    $hdr .= "Transfer-Encoding: chu\r\n"
 	} elsif ( $_ eq 'ce-chunked' ) {
@@ -135,6 +168,11 @@ sub make_response {
 	    $hdr .= "Transfer-Encoding: =?UTF-8?B?Y2h1bmtlZAo=?=\r\n";
 	} elsif ( $_ eq 'xte' ) {
 	    $hdr .= "Transfer-Encoding: lalala\r\n";
+	} elsif ( m{^(.*-)chunked$} ) {
+	    my $prefix = $1;
+	    $prefix =~s{-}{ }g;
+	    $prefix =~s{\bnl\b}{\r\n}g;
+	    $hdr .= "Transfer-Encoding: ${prefix}chunked\r\n";
 	} else {
 	    die $_
 	}
